import { deepCopy, getNumList, randomCode } from "@/plugins/utils";

// element ------------------- >
// 页面
const PAGE = {
  type: "PAGE",
  columnGap: 20,
  locators: [],
  globals: [],
  columns: []
};

// 可编辑栏
const COLUMN = {
  type: "COLUMN",
  x: "",
  y: "",
  w: "",
  h: "",
  isFull: false, // 是否已经填满元素
  elements: []
};
// 定位点
const LOCATOR = {
  type: "LOCATOR",
  x: "",
  y: "",
  w: "",
  h: ""
};

// card-head
const CARD_HEAD_PROP = {
  type: "CARD_HEAD",
  x: 0,
  y: 0,
  w: 0,
  h: 0,
  schoolName: "",
  cardName: "",
  aOrB: true,
  aOrBType: "fill", // fill:手动填涂,auto:自动条码
  missAndFill: true,
  writeSign: true,
  examNumberStyle: "auto", // auto:自动条码, empty:手动条码, fill:手动填涂
  businessParams: [],
  noticeHead: [],
  columnNumber: 2,
  isSimple: false, // 是否是简化形式
  sign: "head"
};

const TOPIC_HEAD_PROP = {
  type: "TOPIC_HEAD",
  x: 0,
  y: 0,
  w: 0,
  h: 60,
  content: "",
  typeName: "",
  sign: "objective" // objective:客观题,subjective:主观题
};

// editable element ------------------- >
// 横线
const LINE_HORIZONTAL_PROP = {
  type: "LINE_HORIZONTAL",
  x: 0,
  y: 0,
  w: 300,
  h: 10,
  sign: "",
  bold: "1px",
  color: "#000000",
  style: "solid"
};
// 竖线
const LINE_VERTICAL_PROP = {
  type: "LINE_VERTICAL",
  x: 0,
  y: 0,
  w: 10,
  h: 300,
  sign: "",
  bold: "1px",
  color: "#000000",
  style: "solid"
};

// 方框
// const RECT_PROP = {
//   type: "RECT",
//   x: 0,
//   y: 0,
//   w: 100,
//   h: 100,
//   sign: "",
//   bold: "1px",
//   backgroundColor: "",
//   borderStyle: "solid"
// };

// 文本
const TEXT_PROP = {
  type: "TEXT",
  x: 0,
  y: 0,
  w: 200,
  h: 50,
  sign: "",
  fontWeight: 400,
  fontFamily: "宋体",
  fontSize: "14px",
  color: "#000",
  content: [
    {
      type: "text",
      content: "样例内容"
    }
  ]
};

// 条形码
// const BARCODE_PROP = {
//   type: "BARCODE",
//   x: 0,
//   y: 0,
//   w: 300,
//   h: 60,
//   sign: "",
//   rotation: 0,
//   bold: "",
//   backgroundColor: "",
//   borderStyle: "solid",
//   content: []
// };

// 图片
const IMAGE_PROP = {
  type: "IMAGE",
  x: 0,
  y: 0,
  w: 150,
  h: 100,
  sign: "",
  borderColor: "",
  borderStyle: "",
  content: []
};

// 选择题
const FILL_QUESTION_PROP = {
  type: "FILL_QUESTION",
  x: 0,
  y: 0,
  w: 0,
  h: 230,
  sign: "objective",
  topicName: "",
  startNumber: 1,
  questionsCount: 10,
  optionCount: 5,
  questionCountPerGroup: 5,
  optionDirection: "horizontal",
  questionGap: 10,
  groupGap: 80,
  optionGap: 10,
  isSelect: true,
  isMultiply: false,
  fontSize: "14px"
};

// 填涂框
// const FILL_AREA_PROP = {
//   type: "FILL_AREA",
//   x: 0,
//   y: 0,
//   w: 100,
//   h: 30,
//   sign: "objective",
//   optionCount: 2,
//   optionDirection: "horizontal",
//   optionGap: 20
// };

// 填空题
const FILL_LINE_PROP = {
  type: "FILL_LINE",
  x: 0,
  y: 0,
  w: 0,
  h: 180,
  sign: "subjective",
  topicName: "",
  startNumber: 1,
  questionsCount: 4,
  questionNumberPerLine: 2,
  lineNumberPerQuestion: 1,
  numberPre: ""
};

// 解答题
// 由多个小题组成，每个小题都是一种元件
const EXPLAIN_PROP = {
  type: "EXPLAIN",
  sign: "subjective",
  topicName: "",
  startNumber: 1,
  questionsCount: 1
};
// 解答题-小题
const EXPLAIN_CHILDREN_PROP = {
  type: "EXPLAIN_CHILDREN",
  x: 0,
  y: 0,
  w: 0,
  h: 425,
  sign: "subjective",
  // 小题序号
  explainNumber: 0,
  // 每一个解答题小题都可以包含其他基础元件，这些基础元件都用绝对定位
  elements: [],
  // 解答题整体信息，EXPLAIN_PROP
  parent: {}
};
// tip属性存在时的条件：parent:大题的小题，container:题目内的子元素

// 作文题
const COMPOSITION_PROP = {
  type: "COMPOSITION",
  x: 0,
  y: 0,
  w: 0,
  h: 350,
  sign: "subjective",
  topicName: "",
  lineCount: 5,
  // 每一个作文题都可以包含其他基础元件，但这些基础元件都用相对定位
  elements: []
};

// available infos
const EDITABLE_ELEMENT = {
  LINE_HORIZONTAL_PROP,
  LINE_VERTICAL_PROP,
  TEXT_PROP,
  IMAGE_PROP
};

const EDITABLE_TOPIC = {
  FILL_QUESTION_PROP,
  FILL_LINE_PROP,
  EXPLAIN_PROP,
  COMPOSITION_PROP
};

const ELEMENT_INFOS = {
  LINE_HORIZONTAL: {
    name: "横线",
    icon: "el-icon-minus"
  },
  LINE_VERTICAL: {
    name: "竖线",
    icon: "el-icon-minus"
  },
  RECT: {
    name: "方框",
    icon: "el-icon-full-screen"
  },
  TEXT: {
    name: "文本",
    icon: "el-icon-tickets"
  },
  BARCODE: {
    name: "条形码",
    icon: "el-icon-c-scale-to-original"
  },
  IMAGE: {
    name: "图片",
    icon: "el-icon-picture-outline"
  },
  FILL_QUESTION: {
    name: "客观题",
    icon: "el-icon-s-operation"
  },
  FILL_AREA: {
    name: "填涂框",
    icon: "el-icon-edit-outline"
  },
  FILL_LINE: {
    name: "填空题",
    icon: "el-icon-edit-outline"
  },
  EXPLAIN: {
    name: "解答题",
    icon: "el-icon-edit-outline"
  },
  COMPOSITION: {
    name: "作文题",
    icon: "el-icon-edit-outline"
  }
};

const ELEMENT_LIST = Object.values(EDITABLE_ELEMENT).map(option => {
  return {
    ...ELEMENT_INFOS[option.type],
    type: option.type
  };
});

const TOPIC_LIST = Object.values(EDITABLE_TOPIC).map(option => {
  return {
    ...ELEMENT_INFOS[option.type],
    type: option.type
  };
});

const getElementId = () => {
  return `element-${randomCode()}`;
};

// 获取元件默认数据结构
const getElementModel = type => {
  const infos = deepCopy(EDITABLE_ELEMENT[`${type}_prop`.toUpperCase()]);
  infos.id = getElementId();
  return infos;
};
// 获取题型默认数据结构
const getTopicModel = type => {
  const infos = deepCopy(EDITABLE_TOPIC[`${type}_prop`.toUpperCase()]);
  infos.id = getElementId();
  return infos;
};

/**
 *
 * @param {Object} cardConfig 题卡配置信息
 */
const getCardHeadModel = cardConfig => {
  const infos = Object.assign({}, CARD_HEAD_PROP, cardConfig);
  infos.id = getElementId();
  return infos;
};

/**
 *
 * @param {String} content
 * @param {String} type objective:客观题,subjective:主观题
 */
const getTopicHead = (content, type) => {
  const typeName = type === "objective" ? "客观题" : "主观题";
  const element = deepCopy(TOPIC_HEAD_PROP);
  element.sign = type;
  element.typeName = typeName;
  element.content = content;
  element.id = getElementId();
  return element;
};

/**
 *
 * @param {Object} explainModel 解答题model
 */
const getExplainChildren = explainModel => {
  const parent = { ...explainModel };

  let elements = [];

  for (var i = 0; i < explainModel.questionsCount; i++) {
    let child = deepCopy(EXPLAIN_CHILDREN_PROP);
    child.id = getElementId();
    child.w = document.getElementById("column-0-0").offsetWidth;
    child.explainNumber = i + explainModel.startNumber;
    child.parent = parent;

    elements[i] = child;
  }

  return elements;
};

// 创建新页面
const getNewPage = (pageNo, columnNumber = 2) => {
  let npage = deepCopy(PAGE);
  if (columnNumber === 4) {
    npage.columnGap = 10;
  }
  npage.locators = getNumList(3).map((item, index) => {
    return [
      {
        ...LOCATOR,
        id: `locator-${pageNo}-${index}0`
      },
      {
        ...LOCATOR,
        id: `locator-${pageNo}-${index}1`
      }
    ];
  });
  npage.columns = getNumList(columnNumber).map((item, index) => {
    return deepCopy(COLUMN);
  });
  return npage;
};

export {
  getElementModel,
  getCardHeadModel,
  getNewPage,
  getTopicHead,
  getTopicModel,
  getExplainChildren,
  getElementId,
  ELEMENT_LIST,
  TOPIC_LIST
};
